package com.example.client.widgets;

import java.util.List;
import java.util.ArrayList;

import com.example.shared.SimpleData;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.Label;
import com.sencha.gxt.core.client.util.Margins;
import com.google.gwt.editor.client.Editor.Path;
import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.core.client.IdentityValueProvider;
import com.sencha.gxt.data.shared.ListStore;
import com.sencha.gxt.data.shared.PropertyAccess;
import com.sencha.gxt.data.shared.ModelKeyProvider;
import com.sencha.gxt.widget.core.client.Window;
import com.sencha.gxt.widget.core.client.button.TextButton;
import com.sencha.gxt.widget.core.client.event.SelectEvent;
import com.sencha.gxt.widget.core.client.event.SelectEvent.SelectHandler;
import com.sencha.gxt.widget.core.client.grid.Grid;
import com.sencha.gxt.widget.core.client.grid.ColumnModel;
import com.sencha.gxt.widget.core.client.grid.ColumnConfig;
import com.sencha.gxt.widget.core.client.grid.CheckBoxSelectionModel;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.HorizontalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
import com.sencha.gxt.widget.core.client.container.HorizontalLayoutContainer.HorizontalLayoutData;

public class GridMultiLineChooser extends Window 
{
	private  final  int  width          = 320;
	private  final  int  height         = 240;
	private  final  int  CONTROL_height =  32;
	
	private  WidgetMultiline        parent = null;
	private  Grid<SimpleData>       grid   = null;
	private  ListStore<SimpleData>  store  = null;

	public  final  String  BUTTON_width    = "80";
	public  final  String  BUTTON_height   = "24";

	public  final  String  TITLE           = "<span style=\"font-size:10pt;font-family:" +
                                             "'Times New Roman',Tahoma,Georgia;\">Выбор записей</span>";
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public GridMultiLineChooser(WidgetMultiline parent)
	{
		this.parent = parent;
		this.getHeader().setHTML(TITLE);
		this.setModal(true);
		
	    VerticalLayoutContainer vlc = new VerticalLayoutContainer();
	    this.setWidget(vlc);
	    
		HorizontalLayoutContainer controls = createControls();

	    this.setPixelSize(width, height);

	    vlc.add(createGrid(), new VerticalLayoutData(1,  1, new Margins(1, 1, 5, 1)));
	    vlc.add(controls    , new VerticalLayoutData(1, -1, new Margins(0, 5, 0, 0)));
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public GridMultiLineChooser(WidgetMultiline parent, final  int  width, final  int  height)
	{
		this(parent);
		setPixelSize(width, height);
		forceLayout();
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public void setList(List<SimpleData> list)
	{
		store.clear();
		if ((list != null) && !list.isEmpty())
			store.addAll(list);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private HorizontalLayoutContainer createControls()
	{
		HorizontalLayoutContainer hlc = new HorizontalLayoutContainer();
		hlc.setHeight(CONTROL_height);
		
    	TextButton btnOk    = new TextButton("Применить");
    	TextButton btnClose = new TextButton("Закрыть"  );

		btnOk   .setSize(BUTTON_width, BUTTON_height);
		btnClose.setSize(BUTTON_width, BUTTON_height);
		
		hlc.add(new Label(""), new HorizontalLayoutData ( 1,  1));
		hlc.add(btnOk        , new HorizontalLayoutData (-1, -1));
		hlc.add(btnClose     , new HorizontalLayoutData (-1, -1));

		btnOk.addSelectHandler(new SelectHandler(){
			@Override
			public void onSelect(SelectEvent se) {
				onOkClick();
			}
		});
		btnClose.addSelectHandler(new SelectHandler(){
			@Override
			public void onSelect(SelectEvent se) {
				hide();
			}
		});
		return hlc;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void selectRecords()
	{
		for (SimpleData rec : store.getAll())
			rec.setSelected(false);
		for (SimpleData select : grid.getSelectionModel().getSelection()) {
			for (SimpleData rec : store.getAll()) {
				if (rec.getKey() == select.getKey()) {
					rec.setSelected(true);
				}
			}
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void onOkClick()
	{
		selectRecords();
		parent.selectionHandler();
		hide();
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void createStore()
	{
		if (store != null)
			return;
		store = new ListStore<SimpleData>(new ModelKeyProvider<SimpleData>() {
			@Override
			public String getKey(SimpleData item) {
				return String.valueOf(item.getKey());
			}
		});
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private Grid<SimpleData> createGrid()	
	{
		IdentityValueProvider<SimpleData> identity = new IdentityValueProvider<SimpleData>();
		IProperties props = GWT.create(IProperties.class);
	    ColumnConfig<SimpleData, String>  nameCol = new ColumnConfig<SimpleData, String>(props.name  (), 80, "Наименование");

	    CheckBoxSelectionModel<SimpleData> sm = new CheckBoxSelectionModel<SimpleData>(identity);

	    List<ColumnConfig<SimpleData, ?>> listColumns = new ArrayList<ColumnConfig<SimpleData, ?>>();
	    listColumns.add(sm.getColumn());
	    listColumns.add(nameCol  );
	    
	    ColumnModel<SimpleData> cm = new ColumnModel<SimpleData>(listColumns);
	    
	    createStore();
	    grid = new Grid<SimpleData>(store, cm);
	    grid.getView().setForceFit(true);
	    grid.setBorders(true);
	    grid.setSelectionModel(sm);
	    return grid;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	interface IProperties extends PropertyAccess<SimpleData>
	{
		@Path("key")
		ModelKeyProvider<SimpleData> key();

		@Path("key")
		ValueProvider<SimpleData, Integer> id();

		@Path("label")
		ValueProvider<SimpleData, String> name();
		
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
}
